/*
    REFERENCES -- bibliographic software
    Copyright (C) 1995-2007  Volker Kiefel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
*/

#ifndef RE_BROW_H
#define RE_BROW_H

#define BROWSE_RECORDS_MENU \
"[f] first, [l] last, [n] next, [p] previous, [c] current record\n\
[k] go to reference with key number, [#] go to position number\n\
[i] information: show record number, [s] save reference number in BBT-file\n\
[a] show abstract (notes, text) of current record\n\
[q] back to menu `enter, edit, view'"

int re_brow_main(char * bbt_file,char * code);
int re_brow_allocate_memory(void);
int re_brow_free_memory(void);
void browse_menu(void);
void browse_bbt(char * bbt_name);
void j1_rec_zeigen(char * au_zeile, char * desc_zeile, char * showrec, char * ex_au_zeile);
void j2_rec_zeigen(char * au_zeile, char * desc_zeile, char * showrec, char * ex_au_zeile);
void b2_rec_zeigen(char * au_zeile, char * ed_zeile, char * desc_zeile, char * showrec, char * ex_au_zeile);
void b1_rec_zeigen(char * ed_zeile, char * desc_zeile, char * showrec, char * ex_au_zeile);
void b3_rec_zeigen(char * ed_zeile, char * desc_zeile, char * showrec, char * ex_au_zeile);
void m1_rec_zeigen(char * au_zeile, char * desc_zeile, char * showrec, char * ex_au_zeile);
void m2_rec_zeigen(char * au_zeile, char * desc_zeile, char * showrec, char * ex_au_zeile);
int auedstring_ok(char * au_zeile, char * ex_au_zeile);
int descstring_ok(char * desc_zeile, char * ex_au_zeile);
void aued_zeile_zeigen(char * zeile, char * ex_au_zeile, char * showrec);
int show_quell(const char * key, char * shows);


#endif


