/*
    REFERENCES -- bibliographic software
    Copyright (C) 1995-2007  Volker Kiefel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
*/

#ifndef IX01_FUN_H
#define IX01_FUN_H

#include "refs.h"

void mrgeix01(char * srcl_name, char * srcsh_name, char * destin);
int read_ix01(char * datname);
int read_write_ix01(char * datname);
int close_ix01(void);
long locate_ix01(const char * keyval, refspix_rec * recd);
void deleteix01(char * srcl, char * srcsh, char * destin);
void appendix01(char * destin_name, char * src_name);
int ok_appix01(char * destin_name, char * src_name);
long scanseq_ix01(const char * keyval, refspix_rec * recd, char * ix_name);


#endif

