/*
    REFERENCES -- bibliographic software
    Copyright (C) 1995-2007  Volker Kiefel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
*/


#ifndef FL_UT_H
#define FL_UT_H


int programmpfad(const char * programm, char * pfad);
int aktuelles_verzeichnis(char * verz);
int slash_to_backslash(char * zle);
int backslash_to_slash(char * zle);
int dir_kat_schreiben(void);
int dir_kat_lesen(void);
void dir_kat_schliessen(void);
int dir_kat_loeschen(void);
// int verzeichnis_auslesen(const char * suchpfad);
int match_filename_pattern_simple(char * filename, const char * search_pattern);
int verz_auslesen_beg_mit(const char * pfad, const char * maske);
int verzeichnis_auslesen(const char * pfad, const char * maske);
// int datei_suchen(const char * suchpfad, char * dateiname);
int datei_suchen(const char * pfad, const char * maske, char * dateiname);
int datei_suchen_mit_pfad(const char * wildcard_spec,
                          const char * pfad,          
                          char * dateiname_mit_pfad
                          );
long filesize(char * datname);
int iseof(FILE * datei);
int isfile(const char * test_datei_name);
int kopieren(char * quelle, char * ziel);

#endif

