/*
    REFERENCES -- bibliographic software
    Copyright (C) 1995-2007  Volker Kiefel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
*/


#ifndef INITIAL_H
#define INITIAL_H

#define VERSION_INFO "\nREFERENCES v4.3 -- bibliographic software\n\
Copyright (C) 1995-2007  Volker Kiefel\n\
\n\
This program is free software; you can redistribute it and/or modify\n\
it under the terms of the GNU General Public License as published by\n\
the Free Software Foundation; either version 2 of the License, or\n\
(at your option) any later version.\n\
 \n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n\
 \n\
You should have received a copy of the GNU General Public License\n\
along with this program; if not, write to the Free Software\n\
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  \n\
USA\n"

#define REFSRUN_DOLLAR "refsrun.$"

#if defined(REFS_USES_MINGW)
  #define REFS_CFG_NAME "refs.cfg"
#endif
#if defined(REFS_USES_LINUXGCC)
  #define REFS_CFG_NAME ".refscfg"
#endif


int initialisieren(void);
int weiter_initialisieren(void);
int cfg_datei_lesen(void);
int logdatei_oeffnen(void);
int logdatei_schliessen(void);
int logdatei_schreiben(char * text);
int temptextdatei_oeffnen(void);
int temptextdatei_schliessen(void);
int temptextdatei_schreiben(char * text);
int protokollieren(char * text);
void dateien_und_pfade_anzeigen(void);
 
#endif


