/*
    REFERENCES -- bibliographic software
    Copyright (C) 1995-2007  Volker Kiefel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
*/

#include "fd.h"

#ifndef RFD_FUNONE_H
#define RFD_FUNONE_H

int rfdmenu(void);
int textdatei_in_fd(char * formdef_datei_name, 
  char * text_datei_name, 
  long dateiposition,
  int editv);
int datensatz_ini(formdef_rec * one_fd);
long fd_datensatz_auswaehlen(void);
int datensatz_lesen(long rec_zeiger, formdef_rec * one_fd);



#endif

