/*
    REFERENCES -- bibliographic software
    Copyright (C) 1995-2007  Volker Kiefel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
*/

#ifndef BTCH_FUN_H
#define BTCH_FUN_H

int text2bbt(void);
int bbt2text(void);
void copy_all_records(void);
int enternameyear(void);
void sortbbt_recno(void);
void sortbbt_text(void);
int remove_double_but_1st(void);
int write_urid(void);
void extract_range(void);
void remove_double_keys(void);
int extract_text_duplicates(void);
int invertbbt_order(void);
int bbt_set_union(void);
int bbt_set_intersection(void);
int bbt_set_difference(void);

#endif

