#!/usr/bin/nawk -f

#
#    REFERENCES -- bibliographic software
#    Copyright (C) 1995-2007  Volker Kiefel
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#    USA
#

#
# Remove empty lines from text file from text files
#

BEGIN {
  abortscript = 0
  if (ARGC == 1) {
    print "\nUsage:"
    print "emptylines.awk textfile\n"
    abortscript = 1
    exit (0)
  }

  ausgabedatei = "reflist_no_emptylines.txt"

}

{
   a = $0
   if (length(a) > 0) {
     print a > ausgabedatei
   }
}

END {
  if (abortscript==0)
  {
    close (a)
    print "\nNew file ``" ausgabedatei "'' written\n"
  }
}
