/*
    REFERENCES -- bibliographic software
    Copyright (C) 1995-2007  Volker Kiefel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
*/

#ifndef LIST_RFD_H
#define LIST_RFD_H

int list_rfd_main(char * mmodus, int testmodus);

/* begin new 28.11.2006 */
#define LIST_RFD_EXPL1 "\n\
----\n\
In the next steps:\n\n\
  * select the file (*.bbt) with the reference numbers to be processed\n\
  * select the format definition in a list\n\
    - view the first page of the list with [f]\n\
    - browse the list, quit the list with [q]\n\
    - enter the number of the format definition or select [quit] to abort\n\
  * select the reference number format (e.g. [n], <ref-no> or no leading\n\
    reference number, this step is not applicable for the commands `create\n\
    macro' [main-l m] or `test macro' [main-l tm])\n\
  * enter the number assigned to the first reference\n\n\
then the text file will be generated, this step returns to the menu `make\n\
lists of references''.\n\
----\n\
"

#define LIST_RFD_EXPL2 "\n\
----\n\
In the next steps:\n\n\
  * select the file (*.bbt) with the reference numbers to be processed\n\
  * select a format definition file (*.fde)\n\
  * select the reference number format (e.g. [n], <ref-no> or no leading\n\
    reference number)\n\
  * enter the number assigned to the first reference\n\n\
then the text file will be generated, this step returns to the menu `make\n\
lists of references''.\n\
----\n\
"
/* end new 28.11.2006 */

#endif
