/*
    REFERENCES -- bibliographic software
    Copyright (C) 1995-2007  Volker Kiefel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
*/


#include "refs.h"
#include "fd.h"

#ifndef RGLOBAL_H
#define RGLOBAL_H

char wrtxtbu[WRTXTBU_LEN+1];
char kennziffer_global[KENNZIFFER_LEN+1];


char cfgpfad[PFAD_LEN+1];
/* current working directory */
char textpfad[PFAD_LEN+1];
/* the ..\rdb directory below the current working directory  */
char rdbpfad[PFAD_LEN+1];


journal_rec one_journal_rec;
desc_rec one_desc_rec;
buch_rec one_buch_rec;
quell_rec one_quell_rec;
refspix_rec ix01rec;
quellpix_rec ix02rec;
keylistpix_rec ix03rec;
keylist_rec one_stichw_rec;
abstr_rec one_abstr_rec;
abstrpix_rec ix04rec;
author_rec one_aued_rec;
formdef_rec fd;



char journal_pix_name[PFAD_LEN+1];
char journal_name[PFAD_LEN+1];
char desc_name[PFAD_LEN+1];
char book_name[PFAD_LEN+1];
char autorjl_name[PFAD_LEN+1];
char autorbu_name[PFAD_LEN+1];
char quell_pix_name[PFAD_LEN+1];
char quell_name[PFAD_LEN+1];
char stichw_name[PFAD_LEN+1];
char stichw_pix_name[PFAD_LEN+1];
char abstr_pix_name[PFAD_LEN+1];
char abstr_dat_name[PFAD_LEN+1];
char formdef_name[PFAD_LEN+1];
char dir_kat_name[PFAD_LEN+1];
char log_name[PFAD_LEN+1];
char tempt_name[PFAD_LEN+1]; /* temporary text file (fptemptext) */
char cfg_name[PFAD_LEN+1];
char refsrun_name[PFAD_LEN+1];

char abst_text_temp[PFAD_LEN+1];

char kennziff_temp_name[PFAD_LEN+1];

/* configuration data */
config_set one_config_set;


FILE * fpjournal;
FILE * fpdesc;
FILE * fpbuch;
FILE * fpautorjl;
FILE * fpautorbu;
FILE * fparchiv;
FILE * fpquell;
FILE * fpsource;
FILE * fpstichw;
FILE * fpformdef;
FILE * fpabs;
FILE * fplog;
FILE * fptemptext;


#endif

