/*
    REFERENCES -- bibliographic software
    Copyright (C) 1995-2007  Volker Kiefel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
*/


#ifndef LIST_BIB_H
#define LIST_BIB_H

int list_bib_main(char * code);
/* begin new 3.12.2006 */
#define LIST_BIB_EXPL1 "\n\
----\n\
In the next steps:\n\n\
  * select the file (*.bbt) with the reference numbers to be processed\n\
  * select the reference number format (original reference numbers or new\n\
    reference numbers starting with the number to be defined in the next step)\n\
  * enter the number assigned to the first reference (optional, depends upon\n\
    the selection made in the previous step)\n\n\
then the text file (.bib) will be generated, this step returns to the menu\n\
``make lists of references''.\n\
----\n\
"
/* end new 3.12.2006 */

#endif
